import {
  changePrimary,
  useAppDispatch,
  useGraphQueryResult,
} from '../../data-access/store';
import { useTheme } from '@mui/material';
import React, { LegacyRef, useEffect, useRef, useState } from 'react';
import styled from 'styled-components';
import ReactJSONView from 'react-json-view';
import styles from './nodelinkvis.module.scss';
import * as PIXI from 'pixi.js';
import { GraphType, LinkType, NodeType } from './Types';
import NodeLinkViewModel from './NodeLinkViewModel';
import ResultNodeLinkParserUseCase from './ResultNodeLinkParserUseCase';
import VisConfigPanelComponent from '../shared/VisConfigPanel/VisConfigPanel';
import NodeLinkConfigPanelComponent from './config-panel/NodeConfigPanelComponent';
import { use } from 'cytoscape';

interface Props {
  loading?: boolean;
  // currentColours: any;
}

const Div = styled.div`
  background-color: red;
  font: 'Arial';
`;

/** Return default radius */
function Radius() {
  return 1;
}

/** This is the interface of the NodeLinkComponentState. */
export interface NodeLinkComponentState {
  graph: GraphType;
  width: number;
  height: number;
  stage: PIXI.Container;
  links: PIXI.Graphics;
  simulation: d3.Simulation<NodeType, LinkType>;
  myRef: React.RefObject<HTMLDivElement>;
  renderer: PIXI.IRenderer;
  dragOffset: any;
  panOffset: any;
  scalexy: number;
}

export const NodeLinkVis = React.memo((props: Props) => {
  const myRef = useRef<HTMLDivElement>(null);
  const nodeLinkViewModelRef = useRef<NodeLinkViewModel>();

  // const [state, setState] = useState<NodeLinkComponentState>({
  //   graph: nodeLinkViewModel.graph,
  //   width: nodeLinkViewModel.width,
  //   height: nodeLinkViewModel.height,
  //   stage: nodeLinkViewModel.stage,
  //   links: nodeLinkViewModel.links,
  //   simulation: nodeLinkViewModel.simulation,
  //   myRef: nodeLinkViewModel.myRef,
  //   renderer: nodeLinkViewModel.renderer,
  //   panOffset: nodeLinkViewModel.panOffset,
  //   dragOffset: nodeLinkViewModel.dragOffset,
  //   scalexy: nodeLinkViewModel.scalexy,
  // });

  const graphQueryResult = useGraphQueryResult();
  const dispatch = useAppDispatch();
  const theme = useTheme();
  console.log('update nodelink');

  useEffect(() => {
    console.log('update nodelink useEffect', graphQueryResult);
    nodeLinkViewModelRef?.current?.consumeMessageFromBackend(graphQueryResult);
    nodeLinkViewModelRef?.current?.startSimulation();
    nodeLinkViewModelRef?.current?.selectD3Elements();
  }, [graphQueryResult]);

  useEffect(() => {
    console.log('loaded NodeLinkVis');
    const resultNodeLinkParserUseCase = new ResultNodeLinkParserUseCase();
    const nodeLinkViewModel = new NodeLinkViewModel(
      resultNodeLinkParserUseCase,
      graphQueryResult,
      myRef?.current?.clientWidth || 1000,
      myRef?.current?.clientHeight || 1000,
      myRef,
      theme
    );
    nodeLinkViewModelRef.current = nodeLinkViewModel;

    // Attach Viewmodel Observer
    // nodeLinkViewModel.attachView(this);

    window.addEventListener('resize', nodeLinkViewModel.handleResize);

    // Init simulation
    nodeLinkViewModel.startSimulation();

    // Init the graph, with node radius 5
    // nodeLinkViewModel.radius = Radius();
    // nodeLinkViewModel.consumeMessageFromBackend(graphQueryResult);
    // nodeLinkViewModel.SetNodeGraphics(graphQueryResult, Radius());

    // Select d3 elements for zooming, panning and dragging
    nodeLinkViewModel.selectD3Elements();

    nodeLinkViewModel.simulation.on('tick', nodeLinkViewModel.ticked);

    // nodeLinkViewModel.subscribeToQueryResult();
    // nodeLinkViewModel.setThisVisAsExportable();
    // nodeLinkViewModel.subscribeToAnalyticsData();
  }, [myRef]);

  useEffect(() => {
    return () => {
      console.log('unloaded NodeLinkVis');

      if (nodeLinkViewModelRef?.current) {
        // Detach Viewmodel Observer
        // nodeLinkViewModel.detachView();
        window.removeEventListener(
          'resize',
          nodeLinkViewModelRef.current.handleResize
        );
      }

    };
  }, []);

  const loading = props.loading;

  return (
    <>
      {/* <input
        onChange={(v) =>
          dispatch(changePrimary({ main: v.currentTarget.value }))
        }
        type="color"
        name="head"
        value={theme.palette.primary.main}
      /> */}
      {loading && (
        <div
          style={{
            marginTop: '40px',
            paddingLeft: '30px',
          }}
        >
          <svg
            // className={styles.rotating}
            xmlns="http://www.w3.org/2000/svg"
            viewBox="0 0 50 50"
          >
            <circle
              // className={styles.ring}
              cx="25"
              cy="25"
              r="20"
            ></circle>
            <circle
              // className={styles.ball}
              cx="25"
              cy="5"
              r="3.5"
            ></circle>
          </svg>
        </div>
      )}
      {!loading && (
        <div
          className="viscontainer"
          style={{
            overflowY: 'auto',
            width: '100%',
            height: '100%',
          }}
        >
          <div
            style={{
              width: '100%',
              height: '100%',
              overflow: 'hidden',
              backgroundColor: '#ffffff', // + props.currentColours.visBackground, TODO
            }}
            className="renderer"
            ref={myRef}
          ></div>
          <VisConfigPanelComponent>
            {/* <NodeLinkConfigPanelComponent
               graph={this.state.graph}
               nlViewModel={this.nodeLinkViewModel}
            /> */}
            {/*</VisConfigPanelComponent>*/}
            {/*<VisConfigPanelComponent isLeft>*/}
            {/*  <AttributesConfigPanel nodeLinkViewModel={this.nodeLinkViewModel} />*/}
          </VisConfigPanelComponent>
        </div>
      )}
    </>
  );
});
NodeLinkVis.displayName = 'NodeLinkVis';

export default NodeLinkVis;
